package applications;

public class affine implements transformation, Cloneable {

  private double[][] matrix = { { 1, 0 }, { 0, 1 } };
  private double[] translation = { 0, 0 };
  
  public void transform(point p) {
    double help = matrix[0][0]*p.x + matrix[0][1]*p.y + translation[0];
    p.y = matrix[1][0]*p.x + matrix[1][1]*p.y + translation[1];
    p.x = help;
  }

/** The identical transformation. */
  public affine() { }
  
  private affine(double m11, double m12, double m21, double m22, double b1, double b2) {
    matrix[0][0] = m11;
    matrix[0][1] = m12;
    matrix[1][0] = m21;
    matrix[1][1] = m22;
    translation[0] = b1;
    translation[1] = b2;
  }
  
/** Compose <code>this</code> with another affine transformation.
  * If <code>this</code> represents a transformation <em>f</em> and
  * <code>a</code> represents <em>g</em> then, after having
  * executed <code>composeWith(a)</code> <code>this</code> will
  * represent the transformation <em>h</em> so that <em>h(x)=g(f(x))</em>.
  */
  public void composeWith(affine a) {
    double help = a.translation[0] + a.matrix[0][0] * translation[0] + a.matrix[0][1] * translation[1];
    translation[1] = a.translation[1] + a.matrix[1][0] * translation[0] + a.matrix[1][1] * translation[1];
    translation[0] = help;
    double[][] newMatrix = new double[2][2];
    newMatrix[0][0] = a.matrix[0][0] * matrix[0][0] + a.matrix[0][1] * matrix[1][0];
    newMatrix[0][1] = a.matrix[0][0] * matrix[0][1] + a.matrix[0][1] * matrix[1][1];
    newMatrix[1][0] = a.matrix[1][0] * matrix[0][0] + a.matrix[1][1] * matrix[1][0];
    newMatrix[1][1] = a.matrix[1][0] * matrix[0][1] + a.matrix[1][1] * matrix[1][1];
    matrix = newMatrix;
  }
  
/** Returns a rotation about the origin by a given angle expressed in radians. */
  public static affine rotation(double angle) {
    double s = Math.sin(angle);
    double c = Math.cos(angle);
    return new affine(c, -s, s, c, 0,0);
  }
  
/** Returns a scaling along the x- and y-axis.
  */
  public static affine scaling(double scaleX, double scaleY) {
    return new affine(scaleX,0,0,scaleY,0,0);
  }
  
/** Returns a translation by a given vector. */
  public static affine translation(double transX, double transY) {
    return new affine(1,0,0,1,transX,transY);
  }

/** Returns a linear transformation determined by the entries of a 2x2 matrix given as
  * arguments.
  * The first two arguments correspond to the top row, the last two to the bottom row
  * of the matrix.
  */
  public static affine matrix(double m11, double m12, double m21, double m22) {
    affine result = new affine();
    result.matrix[0][0] = m11;
    result.matrix[0][1] = m12;
    result.matrix[1][0] = m21;
    result.matrix[1][1] = m22;
    return result;
  }

/** Returns the linear transformation determined by two points and their images.
  * The two points must be linearly independent. If this is not the case,
  * <code>null</code> is returned.
  */
  public static affine linear(double a, double b, double a2, double b2,
                                                             double c, double d, double c2, double d2) {
    double[][] eqSys = {{0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}};
    eqSys[0][0] = eqSys[1][2] = a;
    eqSys[0][1] = eqSys[1][3] = b;
    eqSys[2][0] = eqSys[3][2] = c;
    eqSys[2][1] = eqSys[3][3] = d;
    eqSys[0][4] = a2; eqSys[1][4] = b2; eqSys[2][4] = c2; eqSys[3][4] = d2;
    if (eqSys[0][0] == 0) {
      swap(eqSys, 0, 2); 
      swap(eqSys, 1, 3);
    }
    double det = eqSys[0][0]*eqSys[2][1] - eqSys[0][1]*eqSys[2][0];
    if (det == 0) return null;
    double m12 = (eqSys[0][0] * eqSys[2][4] - eqSys[2][0] * eqSys[0][4]) / det;
    double m11 = (eqSys[0][4] - eqSys[0][1] * m12)/eqSys[0][0];
    double m22 = (eqSys[0][0] * eqSys[3][4] - eqSys[2][0] * eqSys[1][4]) / det;
    double m21 = (eqSys[1][4] - eqSys[0][1] * m22)/eqSys[0][0];
    return matrix(m11, m12, m21, m22);
  }
  
  private static void swap(double[][] matrix, int line1, int line2) {
    double[] help = (double[])matrix[line1].clone();
    matrix[line1] = matrix[line2];
    matrix[line2] = help;
  }

/** Returns the affine transformation determined by three points and their images.
  * The three points must be pairwise linearly independent. If this is not the case,
  * <code>null</code> is returned.
  */
  public static affine affine(double a, double b, double a2, double b2,
                                                             double c, double d, double c2, double d2,
                                                             double e, double f, double e2, double f2) {
    affine result = translation(-e,-f);
    a -= e; b -= f; c -= e; d -= f;
    a2 -= e2; b2 -= f2; c2 -= e2; d2 -= f2;
    affine linearPart = linear(a, b, a2, b2, c, d, c2, d2);
    if (linearPart == null) return null;
    result.composeWith(linearPart);
    result.composeWith(translation(e2,f2));
    return result;
  }

/** Returns the orientation-preserving similarity determined by two points and
  * their images.
  * The two points must not be identical.  Otherwise, <code>null</code> is returned.
  */
  public static affine similarity(double a, double b, double a2, double b2,
                                                             double c, double d, double c2, double d2) {
    a -= c; b -= d; a2 -= c2; b2 -= d2;
    double len1 = Math.sqrt(a*a+b*b);
    double len2 = Math.sqrt(a2*a2+b2*b2);
    if (len1 == 0) return null;
    if (len2 == 0) {
      affine result = scaling(0,0);
      result.composeWith(translation(c2, d2));
      return result;
    }
    affine result = translation(-c,-d);
    result.composeWith(scaling(len2/len1,len2/len1));
    result.composeWith(rotation(Math.atan2(b2/len2,a2/len2) - Math.atan2(b/len1,a/len1)));
    result.composeWith(translation(c2,d2));
    return result;
  }
  
  public Object clone() {
    try {
      affine result = (affine)super.clone();
      result.matrix = (double[][])matrix.clone();
      result.translation = (double[])translation.clone();
      return result;
    }
    catch (CloneNotSupportedException e) { throw new InternalError(); }
  }

}
