package applications.collages;

import java.awt.*;
import java.util.*;
import applications.*;

public class bezierPart extends part {

  public static int filledCurve = 0;
  public static int curve = 1;
  public static int closedCurve = 2;

  private int type;
  private point[][] points = new point[0][3];
  private point[] polyPoints = null;
  private double resolution = 0;
  
  public bezierPart(int type) {
    this.type = type;
  }
  
  public int getType() { return type; }
  
  public void addPoint(point left, point p, point right) {
    point[][] newPoints = new point[points.length + 1][3];
    System.arraycopy(points, 0, newPoints, 0, points.length);
    newPoints[points.length][0] = left;
    newPoints[points.length][1] = p;
    newPoints[points.length][2] = right;
    points = newPoints;
    polyPoints = null;
  }
  
  public void transform(transformation t) {
    for (int i = 0; i < points.length; i++) {
      for (int j = 0; j < 3; j++) t.transform(points[i][j]);
    }
    polyPoints = null;
  }
  
  public point[][] getPoints() { return points; }
  
  public point[] toPolyline(double resolution) {
    if (polyPoints == null || resolution != this.resolution) {
      this.resolution = resolution;
      Vector collect = new Vector();
      collect.addElement(points[0][1]);
      bezierSegment seg = new bezierSegment();
      for (int i = 0; i < points.length-1; i++) {
        seg.setPoints(points[i][1], points[i][2], points[i+1][0], points[i+1][1]);
        seg.toPolyline(resolution, collect);
      }
      if (type == filledCurve || type == closedCurve) {
        seg.setPoints(points[points.length-1][1], points[points.length-1][2], points[0][0], points[0][1]);
        seg.toPolyline(resolution, collect);
      }
      polyPoints = new point[collect.size()];
      collect.copyInto(polyPoints);
    }
    return polyPoints;
  }

  public double[] bounds() {
    double[] bounds = new double[4];
    bounds[0] = bounds[2] = points[0][0].x;
    bounds[1] = bounds[3] = points[0][0].y;
    for (int i = 0; i < points.length; i++) {
      for (int j = 0; j < 3; j++) {
        point p = points[i][j];
        if (p.x < bounds[0]) bounds[0] = p.x;
        if (p.y < bounds[1]) bounds[1] = p.y;
        if (p.x > bounds[2]) bounds[2] = p.x;
        if (p.y > bounds[3]) bounds[3] = p.y;
      }
    }
    return bounds;
  }    
  
  public Object clone() throws CloneNotSupportedException {
    bezierPart result = (bezierPart)super.clone(); 
    result.points = new point[points.length][3];
    for (int i = 0; i < points.length; i++) {
      for (int j = 0; j < 3; j++) result.points[i][j] = new point(points[i][j].x, points[i][j].y);
    }
    result.polyPoints = null;
    return result;
  }

}
