package applications.collages;

import java.awt.*;
import java.util.*;
import util.*;
import applications.*;

public class collage implements Cloneable {

  private list parts = new list();
  private list end = parts;
  
  public void add(part p) {
    end.append(p);
    end = end.tail();
  }
  
  public void unionWith(collage c) {
    if (!c.parts.isEmpty()) {
      parts.concat(c.parts);
      end = c.end;
    }
  }
  
  public void transform(transformation t) {
    list l = parts;
    while (!l.isEmpty()) {
      ((part)l.head()).transform(t);
      l = l.tail();
    }
  }
  
  public Enumeration parts() {
    return new partEnumeration();
  }
  
  public void changeColours(ccOperation op) {
    list l = parts;
    while (!l.isEmpty()) {
      ((part)l.head()).changeColour(op);
      l = l.tail();
    }
  }
  
  public double[] bounds() {
    double[] result = new double[4];
    list l = parts;
    if (l.isEmpty()) {
      result[0] = result[1] = result[2] = result[3] = 0;
    }
    else {
      result[0] = result[1] = Double.POSITIVE_INFINITY;
      result[2] = result[3] = Double.NEGATIVE_INFINITY;
    }
    while (!l.isEmpty()) {
      double[] partBounds = ((part)l.head()).bounds();
      if (partBounds[0] < result[0]) result[0] = partBounds[0];
      if (partBounds[1] < result[1]) result[1] = partBounds[1];
      if (partBounds[2] > result[2]) result[2] = partBounds[2];
      if (partBounds[3] > result[3]) result[3] = partBounds[3];
      l = l.tail();
    }
    return result;
  }
  
  public Object clone() throws CloneNotSupportedException {
    collage result = (collage)super.clone();
    result.parts = result.end = new list();
    list l = parts;
    while (!l.isEmpty()) {
      result.add((part)((part)l.head()).clone());
      l = l.tail();
    }
    return result;
  }
  
  private class partEnumeration implements Enumeration {
    private list next = parts;
    public boolean hasMoreElements() { return !next.isEmpty(); }
    public Object nextElement() {
      Object result = next.head();
      next = next.tail();
      return result;
    }
  }

}
