package applications.collages;

import java.util.*;
import java.awt.*;
import parsers.*;
import terms.*;
import algebras.*;
import applications.*;

public class gridAlgebra extends collageAlgebra {

  private int gridSize;
  private static double squareSize = 20;
  private static String squareName = "sq";
  private static polyPart square = new polyPart(polyPart.filledPolygon);
  static {
      square.addPoint(new point(-squareSize,-squareSize));
      square.addPoint(new point(squareSize,-squareSize));
      square.addPoint(new point(squareSize,squareSize));
      square.addPoint(new point(-squareSize,squareSize));
    }
  private static collage unknownConstant = new collage();
  static {
    try {
      polyPart gray = (polyPart)square.clone();
      gray.setColour((float)(200.0/255.0), (float)(200.0/255.0), (float)(200.0/255.0));
      unknownConstant.add(gray);
      polyPart p = new polyPart(polyPart.polygon);
      p.addPoint(new point(-.8*squareSize,-.8*squareSize));
      p.addPoint(new point(.8*squareSize,-.8*squareSize));
      p.addPoint(new point(.8*squareSize,.8*squareSize));
      p.addPoint(new point(-.8*squareSize,.8*squareSize));
      unknownConstant.add(p);
      polyPart arrow = new polyPart(polyPart.filledPolygon);
      arrow.addPoint(new point(-.8*squareSize,-.8*squareSize));
      arrow.addPoint(new point(-.8*squareSize+squareSize/15, -.8*squareSize+squareSize/5));
      arrow.addPoint(new point(-.8*squareSize-squareSize/15, -.8*squareSize+squareSize/5));
      unknownConstant.add(arrow);
    } catch (CloneNotSupportedException e) { }
  }
  private static collage sq = new collage();
  static {
      sq.add(square);
    }
  
  public gridAlgebra() { }
  
  public void setGridSize(int n) { gridSize = n; }
  
  protected Object extendBy(symbol s) {
    Object op = decodeOp(s.toString());
    int rank;
    if (op == null || op instanceof collage) rank = 0;
    else if (op instanceof ccOperation) rank = 1;
    else rank = ((affine[])op).length;
    if (op == null || rank != s.rank()) {
      if (s.rank() == 1) {
        affine[] result = new affine[1];
        result[0] = new affine();
        return result;
      }
      else if (s.rank() == 0) { return unknownConstant; }
    }
    if (rank != s.rank()) return null;
    else return op;
  }
  
  public Object decodeOp(String name) {
    ccOperation ccOp = ccOperation.decode(name);
    if (ccOp != null) return ccOp;
    if (squareName.equals(name)) return sq;
    Vector result = new Vector();
    int i = 0;
    do {
      while (name.length() > 0 && name.charAt(0) == '-') name = name.substring(1);
      if (name.length() == 0) break;
      if (name.charAt(0) < '0' || name.charAt(0) > '9') return null;
      int j;
      for (j = 0; name.length() > j && name.charAt(j) >= '0' && name.charAt(j) <= '9'; j++) {}
      int sqIndex = 0;
      try { sqIndex = Integer.parseInt(name.substring(0,j)); }
      catch (NumberFormatException e) { throw new InternalError(e.toString()); }
      name = name.substring(j);
      if ( sqIndex >= gridSize * gridSize || name.length() < 3) return null;
      affine aff = transformation(sqIndex, name.substring(0,3));
      if (aff == null) return null;
      else result.addElement(aff);
      name = name.substring(3);
    } while (true);
    affine[] aff = new affine[result.size()];
    result.copyInto(aff);
    return aff;
  }

  
  private affine transformation(int sqNumber, String description) {
    double rot;
    int mirror;
    if ("abc".equals(description)) { rot = 0; mirror = 1; }
    else if ("bcd".equals(description)) { rot = Math.PI/2; mirror = 1; }
    else if ("cda".equals(description)) { rot = Math.PI; mirror = 1; }
    else if ("dab".equals(description)) { rot = Math.PI/2*3; mirror = 1; }
    else if ("adc".equals(description)) { rot = Math.PI/2*3; mirror = -1; }
    else if ("dcb".equals(description)) { rot = Math.PI; mirror = -1; }
    else if ("cba".equals(description)) { rot = Math.PI/2; mirror = -1; }
    else if ("bad".equals(description)) { rot = 0; mirror = -1; }
    else return null;
    int squarePosX = sqNumber % gridSize;
    int squarePosY = sqNumber / gridSize;
    double size = 2 * squareSize / gridSize;
    double transX = -squareSize + squarePosX * size;
    double transY = -squareSize + squarePosY * size;
    affine result = affine.scaling(mirror / (double)gridSize, 1 / (double)gridSize);
    result.composeWith(affine.rotation(rot));
    result.composeWith(affine.translation(transX + size/2, transY + size/2));
    return result;
  }

/** Initialize a <code>gridAlgebra</code> by reading its definition from a stream.
  * @see gridAlgebraParser
  * @exception ParseException if an error occurs
  */
  public void parse(ASCII_CharStream stream) throws ParseException {
    gridAlgebraParser parser = new gridAlgebraParser(stream);
    parser.gridAlgebra(this);
  }

  
}
