package applications.lineDrawings;

import algebras.*;
import terms.*;

public  abstract class lineDrawingAlgebra extends extensibleAlgebra {

  private static String vshrink = "v-shrink"; 
  private static String hshrink = "h-shrink";
  private static String shrink = "shrink";
  public static int vtype = 0;
  public static int htype = 1;
  public static int hvtype = 2;
  
  protected Object apply(symbol op, Object[] args) {
    if (args.length == 0) return new lineDrawing();
    lineDrawing result = (lineDrawing)args[0];
    for (int i = 1; i < args.length; i++) result.concat((lineDrawing)args[i]);
    Object o = operationOf(op);
    if (o != null && o instanceof operation) {
      operation shrinkOp = (operation)o;
      result.shrink(shrinkOp.type == vtype ? 1 : shrinkOp.divisor,
                    shrinkOp.type == htype ? 1 : shrinkOp.divisor);
    }
    return result;
  }
  
  protected Object extendBy(symbol s) {
    operation op;
    double divisor;
    String name = s.toString();
    String number;
    if (name.startsWith(vshrink)) {
      op = new operation(vtype);
      number = name.substring(vshrink.length());
    }
    else if (name.startsWith(hshrink)) {
      op = new operation(htype);
      number = name.substring(hshrink.length());
    }
    else if (name.startsWith(shrink)) {
      op = new operation(hvtype);
      number = name.substring(shrink.length());
    }
    else return null;
    int i = 0;
    while (i < number.length() && number.charAt(i) >= '0' && number.charAt(i) <= '9') i++;
    if (i < number.length() && number.charAt(i) == '.') i++;
    while (i < number.length() && number.charAt(i) >= '0' && number.charAt(i) <= '9') i++;
    if (i < number.length()) return null;
    try { divisor = Double.valueOf(number).doubleValue(); }
    catch (NumberFormatException e) { return null; }
    if (divisor < 0) return null;
    else op.divisor = divisor;
    return op;
  }
  
  private class operation {
    public int type;
    public double divisor;
    public operation(int type) { this.type = type; }
  }

}
