package gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import gui.nodeTypes.*;
import terms.*;

public class edge extends Component implements Observer {

  public static Object MOVEMENT = new Object();
  public static Object CHANGE = new Object();

  private static int arrowAngle = 40;
  private static int arrowLength = 15;
  private static int arrowWidth;
    { arrowWidth = (int)(arrowLength * Math.sin(arrowAngle/2)); }
  private Color colour = Color.black;

  public node source, target;
  private Point from, to;
  private Rectangle bounds = new Rectangle();

  public edge(node source, node target) {
    super();
    this.source = source;
    this.target = target;
    source.observeMotionBy(this);
    target.observeMotionBy(this);
    setBounds();
  }
  
  public void setColour(Color c) {
    colour = c;
  }
  
  private void setBounds() {
    from = source.getBorderPoint(target.position);
    to = target.getBorderPoint(source.position);
    if (from != null && to != null) {
      bounds.x = Math.min(from.x, to.x) - arrowWidth;
      bounds.y = Math.min(from.y, to.y) - arrowWidth;
      bounds.width = Math.abs(from.x - to.x) + 1 + 2 * arrowWidth;
      bounds.height = Math.abs(from.y - to.y) + 1 + 2 * arrowWidth;
    }
    else {
      bounds.width = 0;
      bounds.height = 0;
    }
    setBounds(bounds);
  }
  
  public void paint(Graphics g) {
    super.paint(g);
    g.setColor(colour);
    if (from == null || to == null) return;
    int x1 = from.x-bounds.x;
    int y1 = from.y-bounds.y;
    int x2 = to.x-bounds.x;
    int y2 = to.y-bounds.y;
    int angle = (int)(90 + 180/Math.PI*Math.atan2(x2 - x1, y2 - y1));
    g.drawLine(x1, y1, x2, y2);
    g.fillArc(x2 - arrowLength, y2 - arrowLength,
              2*arrowLength, 2*arrowLength, angle - arrowAngle/2, arrowAngle);
  }
  
  public void update(Observable o, Object arg) {
    if (arg == MOVEMENT) { // one of the two nodes has been moved
      setBounds();
      invalidate();
      repaint();
    }
    else if (arg == CHANGE) // change in source node occurred
      ((worksheetNode)target).ping();
    else { // argument is a new term computed by source node
      ((worksheetNode)target).transferCommand(arg, worksheetNode.TERM);
    }
  }
  
}
