package gui.nodeTypes;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import displays.*;
import algebras.*;
import terms.*;
import generators.*;
import gui.*;
import parsers.*;
import util.*;

public class displayNode extends worksheetNode {

  private JFrame window = null;
  private edge semEdge = null;
  private edge inputEdge = null;
  private static int xDist = 4*radius/7;
  private static int delta = radius/7;
  private static int yDist = 4*radius/9;
  private static int [] iconX = { -xDist, -xDist+delta, xDist-delta, xDist,
                                  xDist, xDist-delta, -xDist+delta, -xDist};
  private static int [] iconY = { yDist-delta, yDist, yDist, yDist-delta,
                                  -yDist+delta, -yDist, -yDist, -yDist+delta };

  public displayNode(display disp, String fileName) {
    super(disp, fileName);
    figureX = iconX;
    figureY = iconY;
  }
  
  public int openWindows() {
    return super.openWindows() + (window != null && window.isShowing() ? 1 : 0);
  }
  
  private boolean initWindow() {
    Component c = ((display)contents).visualizer();
    if (c != null) {
      window = new JFrame();
      windowTitle();
      window.getContentPane().setBackground(Color.white);
      window.getContentPane().add(c);
      window.pack();
      Point p = parent().getLocationOnScreen();
      window.setLocation(new Point(p.x + position.x + radius, p.y + position.y + radius));
      return true;
    }
    else return false;
  }
  
  private void windowTitle() {
    if (window != null) {
      if (inputEdge != null) {
        termGeneratorNode source = (termGeneratorNode)inputEdge.source;
        String inputName = source.getName();
        window.setTitle(getName() + " (input: " + inputName + ")");
      }
      else window.setTitle(getName() + " <no input>");
    }
  }        
  
  public boolean acceptsSource(worksheetNode source) {
    if (source instanceof algebraNode) {
      algebra alg = (algebra)((worksheetNode)source).contents;
      return ((display)contents).acceptable(alg);
    }
    return source instanceof termGeneratorNode;
  }
  
  public void sourceEstablished(edge e) {
    worksheetNode source = (worksheetNode)e.source;
    if (source instanceof termGeneratorNode) {
      if (inputEdge != null) parent().deleteEdge(inputEdge);
      inputEdge = e;
      windowTitle();
    }
    else if (source instanceof algebraNode) {
      if (semEdge != null) parent().deleteEdge(semEdge);
      semEdge = e;
    }
    redisplay();
  }
  
  public void redisplay() {
    if (inputEdge != null && semEdge != null) { 
      termGeneratorNode source = (termGeneratorNode)inputEdge.source;
      transferCommand(source.currentTerm(), TERM);
    }
    else transferCommand(null, TERM);
  }
  
  public void open() {
    if (window != null || initWindow()) {
      if (window.isShowing()) {
        window.setVisible(true);
        super.open();
      }
      else window.setVisible(true);
    }
    else super.open();
  }
  
  protected void reinit(parsable newContents) {
    super.reinit(newContents);
    Component c = ((display)contents).visualizer();
    if (window != null) {
      if (c != null) {
        window.getContentPane().removeAll();
        window.getContentPane().add(c);
        window.pack();
      }
      else {
        window.dispose();
        window = null;
      }
    }
  }
  
  public void ping() {
    redisplay();
  }
  
  public void runContents(Object command, int type) {
    if (type == TERM) {
      display d = (display)contents;
      if (semEdge != null) d.setSemantics((algebra)((algebraNode)semEdge.source).contents);
      else d.setSemantics(null);
      d.display((term)command);
    }
    else super.runContents(command, type);
  }
  
  public void sourceDeleted(edge e) {
    if (e == inputEdge) inputEdge = null;
    else if (e == semEdge) {
      semEdge = null;
    }
    windowTitle();
    redisplay();
  }
  
  protected void dispose() {
    if (window != null) {
      window.getContentPane().removeAll();
      window.dispose();
    }
    window = null;
    super.dispose();
  }

}
