package gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import util.*;

public class worksheet extends JFrame implements ActionListener, MouseListener {

  private static String versionNumber = "1.2";
  private static String load = "load";
  private static String add = "add";
  private static String save = "save";
  private static String deleteAll = "clear";
  private static String quit = "quit";
  private static String addComp = "add component";
  private static String[] worksheetCommands = { load, add, save, deleteAll, null, quit };
  private static String[] componentCommands = { addComp };

  private componentGraph graph;
  private PopupMenu popup = new PopupMenu();
  private Point popedUpAt;

  public worksheet() {
    super("TREEBAG " + versionNumber + " worksheet");
    setDefaultCloseOperation(EXIT_ON_CLOSE);
    graph = new componentGraph();
    GridBagLayout layout = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.fill = GridBagConstraints.BOTH;
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.weightx = constraints.weighty = 1.0;
    layout.setConstraints(graph, constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weighty = 0;
    worksheetPanel panel = new worksheetPanel();
    layout.setConstraints(panel, constraints);
    pane().setLayout(layout);
    pane().add(graph);
    pane().add(panel);
    pane().add(popup);
    graph.addMouseListener(this);
    initMenus();
    pack();
    setVisible(true);
  }
  
  public worksheet(String fileName) {
    this();
    graph.addConfig(true, (new File(fileName)).getAbsolutePath());
    pack();
  }
  
  private Container pane() { return getContentPane(); }
  
  private void initMenus() {
    JMenuBar bar = new JMenuBar();
    JMenu menu = new JMenu("Worksheet", true);
    for (int i = 0; i < worksheetCommands.length; i++) {
      if (worksheetCommands[i] == null) {
        menu.addSeparator();
        popup.addSeparator();
      }
      else {
        JMenuItem item = new JMenuItem(worksheetCommands[i]);
        item.addActionListener(this);
        menu.add(item);
        popup.add(new MenuItem(worksheetCommands[i]));
      }
    }
    bar.add(menu);
    popup.addSeparator();
    popup.addSeparator();
    menu = new JMenu("Component", true);
    for (int i = 0; i < componentCommands.length; i++) {
      if (worksheetCommands[i] == null) {
        menu.addSeparator();
        popup.addSeparator();
      }
      else {
        JMenuItem item = new JMenuItem(componentCommands[i]);
        item.addActionListener(this);
        menu.add(item);
        popup.add(new MenuItem(componentCommands[i]));
      }
    }
    bar.add(menu);
    setJMenuBar(bar);
    popup.addActionListener(this);
    bar.setVisible(true);
  }
  
  public void actionPerformed(ActionEvent e) {
    String com = e.getActionCommand();
    if (quit.equals(com)) System.exit(0);
    else if (deleteAll.equals(com)) graph.deleteAllNodes();
    else if (addComp.equals(com)) {
      if (e.getSource() == popup) graph.newNode(popedUpAt);
      else graph.newNode(new Point(graph.getSize().width/2, graph.getSize().height/2));
    }
    else if (save.equals(com)) graph.saveConfig();
    else if (add.equals(com) || load.equals(com)) {
      graph.addConfig(load.equals(com));
      pack();
    }
  }
  
  public void mouseClicked(MouseEvent e) {
  }
  
  public void mousePressed(MouseEvent e) {
    popedUpAt = e.getPoint();
    if (e.isPopupTrigger()) popup.show(graph, popedUpAt.x, popedUpAt.y);
  }
  
  public void mouseReleased(MouseEvent e) {
  }
  
  public void mouseEntered(MouseEvent e) {
  }
  
  public void mouseExited(MouseEvent e) {
  }

  public static void main(String[] args) {
    if (args.length == 0) new worksheet();
    else new worksheet(args[0]);
  }  
  
  private class worksheetBar extends Canvas {
  
    private int height = 3;
  
    public worksheetBar() {
      super();
      setSize(0, height);
    }
    
    public void paint(Graphics g) {
      super.paint(g);
      int width = getSize().width;
      g.setColor(Color.white);
      for (int i = 0; i < height; i++) {
        g.setColor(g.getColor().darker());
        g.drawLine(0,i,width-1, i);
      }
    }
  
  }
    
  private class worksheetPanel extends JPanel {
  
    private TextField progress;
  
    public worksheetPanel() {
      super(new FlowLayout(FlowLayout.LEFT));
      setBorder(BorderFactory.createEtchedBorder());
      progress = new TextField();
      progress.setEditable(false);
      progress.setBackground(Color.white);
      add(progress);
      graph.setProgressField(progress);
    }
    
  }


}

