package terms;

import java.util.*;
import util.*;
import parsers.*;

/** A finite signature.
  * Having different symbols with the same name is allowed.
  */
public class finiteSignature implements signature {

  private Hashtable symbols = new Hashtable();
  private static Boolean TRUE = new Boolean(true);
  
/** Adds a symbol to a signature.
  */
  public void addSymbol(symbol s) {
    symbols.put(s, TRUE);
  }
  
  public boolean contains(symbol sym) {
    return symbols.get(sym) == TRUE;
  }
  
/** Test whether another signature is disjoint with this one. */
  public boolean disjointWith(signature sig) {
    Enumeration enum = symbols.keys();
    while (enum.hasMoreElements()) {
      if (sig.contains((symbol)enum.nextElement())) return false;
    }
    return true;
  }
  
/** Add all symbols of another finite signature to this one. */
  public void unionWith(finiteSignature sig) {
    Enumeration enum = sig.elements();
    while (enum.hasMoreElements()) addSymbol((symbol)enum.nextElement());
  }
  
/** An enumeration of all the symbols in the signature. */
  public Enumeration elements() { return symbols.keys(); }
  
  public String toString() {
    StringBuffer result = new StringBuffer();
    result.append("{ ");
    Enumeration elem = elements();
    if (elem.hasMoreElements()) {
      symbol sym = (symbol)elem.nextElement();
      result.append(sym.toString() + ":" + sym.rank());
      while (elem.hasMoreElements()) {
        sym = (symbol)elem.nextElement();
        result.append(", " + sym.toString() + ":" + sym.rank());
      }
    }
    result.append(" }");
    return result.toString();
  }

/** Parse a signature definition.
    @exception ParseException if an error occurs
  */
  public void parse(ASCII_CharStream stream) throws ParseException {
    setParser parser = new setParser(stream);
    parser.set(new cp(new symbolParser(stream)));
  }
  
  private class cp implements componentParser {
    private symbolParser symbols;
    public cp(symbolParser symbols) { this.symbols = symbols; }
    public void component() throws ParseException { addSymbol(symbols.symbol()); }
  }

}
